#!/usr/bin/python

# TODO: check if there was a -dbgsym version of the package included in stretch
# TODO: check if that package has a Replaces for the dbg package.

# TODO: Where do we get apt informatoin for stretch?
